extern u8 Image$$EWRAM$$ZI$$Limit[];	//first available chunk of memory
extern u8 Load$$DTCM$$Base[];
extern u32 Image$$DTCM$$Length;

#define NES_SRAM 0x23f8800

#define VRAM_ABCD (*(vu32*)0x4000240)
#define VRAM_EFG (*(vu32*)0x4000244)
#define VRAM_HI (*(vu16*)0x4000248)

#ifdef ARM9
	#define DTCM 0x2800000
	#undef IRQ_HANDLER
	#define IRQ_HANDLER (*(VoidFunctionPointer *)(DTCM+0x3FFC))
	#undef VBLANK_INTR_WAIT_FLAGS
	#define VBLANK_INTR_WAIT_FLAGS (*(u32*)(DTCM+0x3FF8))
#endif

#undef IPC		//IPC_* also in equates.h, keep both updated
#define IPC 0x27fff00
#define IPC_TOUCH_X	(*(vu16*)(IPC+0))
#define IPC_TOUCH_Y	(*(vu16*)(IPC+2))
#define IPC_KEYS	(*(vu32*)(IPC+4))
#define IPC_ALIVE	(*(vu32*)(IPC+8))
#define IPC_TMP0	(*(vu32*)(IPC+12))
#define IPC_TMP1	(*(vu32*)(IPC+16))
#define IPC_APU_WRITE_HEAD (*(vu32*)(IPC+20))
#define IPC_APU_WRITE_TAIL (*(vu32*)(IPC+24))
#define IPC_SLEEP	(*(vu32*)(IPC+28))

#define soundbuff ((u32*)0x27ff800)	//(256 words) see equates.h

#undef KEY_TOUCH
#define KEY_TOUCH 0x10000
#define KEY_CLOSED 0x20000

#define FIFO_WRITEPM 1
#define FIFO_APU_PAUSE 2
#define FIFO_UNPAUSE 3
#define FIFO_SETVOLUME 4
#define FIFO_APU_RESET 5
#define FIFO_HIBERNATE 6

void fifo_send(u32 msgID,u32 msg);

typedef struct {	//rom info from builder
	char name[32];
	u32 filesize;
	u32 flags;
	u32 spritefollow;
	u32 reserved;
} romheader;	

#define SAVESTATESIZE (0x2800+0x3000+96+64+16+32+16+44+48)

//nesmain.c
void play(void);
void recorder_reset(void);

//console.c
#define SUB_CHR 0x6204000
#define SUB_BG 0x6200000
void consoleinit(void);
#define hex8(a,b) hex(a,b,1)
#define hex16(a,b) hex(a,b,3)
#define hex24(a,b) hex(a,b,5)
#define hex32(a,b) hex(a,b,7)
void hex(int offset,int d,int n);
void consoletext(int offset,char *s,int color);
void menutext(int line,char *s,int selected);
void clearconsole(void);
void hideconsole(void);
void showconsole(void);

//subscreen.c
int adjustdisplay(void);
int do_touchcontroller(void);
int savestuff(void);

#define VBLS 6
#define FPS 7
extern u32 debuginfo[];
int debugdump(void);

//romloader.c
extern char romfilename[256];
extern char *romfileext;
void do_rommenu(void);
extern int freemem_start;
#define freemem_end 0x23f8000

//misc.c
typedef struct {
	int offset;
	char *str;
} touchstring;
extern int touchstate, last_x, last_y;
void touch_update(void);
int do_touchstrings(touchstring*,int pushstate);
void load_sram(void);
void save_sram(void);
void write_savestate(void);
void read_savestate(void);
void ARM9sleep(void);


//*.s

//emuflags:
#define NOFLICKER 1
#define ALPHALERP 2
//joyflags
#define B_A_SWAP	0x80000
#define L_R_DISABLE	0x100000
//cartflags
#undef SRAM
#define SRAM 0x02

extern u32 emuflags;		//cart.s
extern u8 cartflags;

extern u32 joyflags;		//io.s
extern u32 joystate;
extern u32 romstart;

extern u8 mapperstate[32];	//6502.s

void EMU_VBlank(void);
void EMU_Run(void);
void initcart(u8 *rom);//,int flags);
void PPU_init(void);
void rescale(u32 scale, u32 start);
void paletteinit(void);		//ppu.s
void NES_reset(void);		//cart.s
int savestate(u32);
int loadstate(u32);